@@ -38,13 +38,17 @@
 	 we wrote the opcodes, they're going to be what we accept. dodgy. */
 
 #define rl_fail1(a) \
-  rl_warn("%s(%d) failed %s", (a), *op, strerror(errno))
+	rl_warn("%s(%d) failed %s", (a), *op, strerror(errno))
 
 #define RL_MIN(a, b) ((a) < (b) ? (a) : (b))
 
 int allow_severity = LOG_INFO;
 int deny_severity = LOG_WARNING;
 
+#ifdef BYTECODE_DEBUG
+static char * bytecode2string(rl_opcode_t op);
+#endif
+
 void run_bytecode(rl_opcode_t *op, struct rl_instance *inst) {
 	if(!op)
 		return;
@@ -52,7 +56,11 @@ void run_bytecode(rl_opcode_t *op, struct rl_instance *inst) {
 		fprintf(stderr, "sched(%d)\n", getpid());
 	do {
 		if(rl_debug > 1)
+#ifdef BYTECODE_DEBUG
+			fprintf(stderr, "op %d: %s\n", *op, bytecode2string(*op));
+#else		
 			fprintf(stderr, "op %d\n", *op);
+#endif			
 		switch(*op) {
 			struct argvtab *argv;
 			struct logtab *log;
@@ -138,8 +146,11 @@ void run_bytecode(rl_opcode_t *op, struct rl_instance *inst) {
 					free(inst->sin);
 				inst->sin = NULL;
 				inst->sinlen = i;
-				if(i)
+				if(i) {
 					inst->sin = (struct sockaddr *)malloc(i);
+					if (!inst->sin)
+						rl_fatal(EX_SOFTWARE, "ABORT - Can't allocate memory");
+				}
 				if((inst->sock = accept(inst->sock, inst->sin, &inst->sinlen)) < 0) {
 					rl_warn("accept(): %s",	strerror(errno));
 					return;
@@ -188,7 +199,17 @@ void run_bytecode(rl_opcode_t *op, struct rl_instance *inst) {
 				fromhost(request);
 
 				if(!hosts_access(request)) {
-					refuse(request);
+					char host_ip[100];
+					struct loglist l;
+
+					/* get address of client */
+					l.type = LOG_SOURCE_IP;
+					l.arg = host_ip;
+					l.len = sizeof(l.arg);
+					loglist_build( inst, &l );
+
+					rl_warn("access for service %s from host %s denied by TCP wrappers", str, host_ip);
+					free( request );
 					run_bytecode(oplisttab_get(k), inst);
 				}
 #endif
@@ -225,7 +246,7 @@ void run_bytecode(rl_opcode_t *op, struct rl_instance *inst) {
 			case OP_RET:
 				return;
 			case OP_EXIT:
-				abort();
+				exit(0);
 				return;
 			case OP_ECHO:
 				argv = argvtab_get(*++op);
@@ -324,3 +345,57 @@ void run_bytecode(rl_opcode_t *op, struct rl_instance *inst) {
 		}
 	} while(op++);
 }
+
+#ifdef BYTECODE_DEBUG
+static char * bytecode2string(rl_opcode_t op)
+{
+	switch(op) {
+	
+		case OP_EXEC    : return "OP_EXEC";
+		case OP_FISH    : return "OP_FISH";
+		case OP_SUID    : return "OP_SUID";
+		case OP_SGID    : return "OP_SGID";
+		case OP_NICE    : return "OP_NICE";
+		case OP_RLIMIT  : return "OP_RLIMIT";
+		case OP_LOG     : return "OP_LOG";
+		case OP_CHROOT  : return "OP_CHROOT";
+		case OP_ACCEPT  : return "OP_ACCEPT";
+		case OP_FORK    : return "OP_FORK";
+		case OP_WRAP    : return "OP_WRAP";
+		case OP_SETCAP  : return "OP_SETCAP";
+		case OP_INITGR  : return "OP_INITGR";
+		case OP_BRANCH  : return "OP_BRANCH";
+		case OP_CLOSE   : return "OP_CLOSE";
+		case OP_BUFCOPY : return "OP_BUFCOPY";
+		case OP_ZERO    : return "OP_ZERO";
+		case OP_RET     : return "OP_RET";
+		case OP_EXIT    : return "OP_EXIT";
+		case OP_ECHO    : return "OP_ECHO";
+		case OP_UP      : return "OP_UP";
+		case OP_DOWN    : return "OP_DOWN";
+		case OP_FROG    : return "OP_FROG";
+		case OP_LSET    : return "OP_LSET";
+		case OP_LCLR    : return "OP_LCLR";
+		case OP_BUFINIT : return "OP_BUFINIT";
+		case OP_BUFREAD : return "OP_BUFREAD";
+		case OP_BUFWRITE: return "OP_BUFWRITE";
+		case OP_RHOOK   : return "OP_RHOOK";
+		case OP_RUNHOOK : return "OP_RUNHOOK";
+		case OP_WHOOK   : return "OP_WHOOK";
+		case OP_WUNHOOK : return "OP_WUNHOOK";
+		case OP_BZ      : return "OP_BZ";
+		case OP_JUMP    : return "OP_JUMP";
+		case OP_BZNEG   : return "OP_BZNEG";
+		case OP_ADD     : return "OP_ADD";
+		case OP_SUB     : return "OP_SUB";
+		case OP_DUP     : return "OP_DUP";
+		case OP_POP     : return "OP_POP";
+		case OP_BUFCLONE: return "OP_BUFCLONE";
+		case OP_BUFFREE : return "OP_BUFFREE";
+		default         : return "UNKNOWN";
+	}				
+};
+
+#endif
+
+/* vim: set ts=2: */
